package ch.epfl.tpp.sol;

import java.util.Iterator;
import java.util.Map;
import java.util.Set;

public class ConcreteInvertedIndex implements InvertedIndex {
    private final CollectionsFactory collectionsFactory;
    private final Set<Document> emptyDocSet;
    private final Map<String, Set<Document>> index;

    public ConcreteInvertedIndex(CollectionsFactory collectionsFactory) {
        this.collectionsFactory = collectionsFactory;
        this.index = collectionsFactory.newMap();
        this.emptyDocSet = collectionsFactory.newSet();
    }
    
    @Override
    public void addDocument(Document d) {
        Iterator<String> it = d.wordsIterator();
        while (it.hasNext()) {
            String w = it.next();
            if (! index.containsKey(w))
                index.put(w, collectionsFactory.<Document>newSet());
            index.get(w).add(d);
        }
    }
    
    @Override
    public Set<Document> documentsWithWord(String word) {
        return index.containsKey(word) ? index.get(word) : emptyDocSet;
    }
}
