package ch.epfl.tpp.sol;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;

public class InvertedIndexBenchmarker {
    private final static int DOCUMENT_WORDS = 7000;
    private final static int DOCUMENT_COUNT = 150;
    
    private static List<Document> createRandomDocuments() {
        Random rng = new Random(2013);
        List<Document> documents = new ArrayList<>(DOCUMENT_COUNT);
        for (int i = 0; i < DOCUMENT_COUNT; ++i) {
            Document d = new RandomDocument(DOCUMENT_WORDS, rng.nextLong());
            documents.add(d);
        }
        return documents;
    }

    public static void main(String[] args) {
        List<Document> documents = createRandomDocuments();
        Set<String> words = new TreeSet<>();
        for (Document d: documents) {
            Iterator<String> wordIt = d.wordsIterator();
            while (wordIt.hasNext())
                words.add(wordIt.next());
        }

        long startTime = System.nanoTime();
        bench(documents, words);
        long endTime = System.nanoTime();
        System.out.printf("Temps total : %.2f\n", (double)(endTime - startTime) / 1e9);
    }

    private static void bench(List<Document> documents, Set<String> words) {
        InvertedIndex ii = new ConcreteInvertedIndex(new CollectionsFactory() {
            @Override
            public <E> Set<E> newSet() {
                return new TreeSet<>();
            }
            @Override
            public <K, V> Map<K, V> newMap() {
                return new TreeMap<>();
            }
            
        });
        for (Document d: documents)
            ii.addDocument(d);
        
        for (String word: words) {
            Set<Document> docs = ii.documentsWithWord(word);
            if (docs.size() == DOCUMENT_COUNT)
                System.out.println("  Mot présent dans tous les documents : "+ word);
        }
    }
}
